@@ -1140,24 +1140,38 @@ WidgetFactory.initLineEdit = function(lineEdit, dimension) {
 
         // TODO: split into submenus (math, greek, ...):
         var symbols = [
-                ["\u00F8", qsTr("Diameter")],
-                ["\u2312", qsTr("Arc")],
-                ["\u00B0", qsTr("Degree")],
-                ["\u00B1", qsTr("Plus/Minus")],
-                ["\u2248", qsTr("Almost equal to")],
-                ["\u2243", qsTr("Asymptotically equal to")],
-                ["\u03C0", qsTr("Pi")],
-                ["\u221A", qsTr("Square root")],
-                ["\u03A6", qsTr("Phi")],
-                ["\u0278", qsTr("phi")],
-                ["\u03C6", qsTr("Alt phi")],
-                ["\u2126", qsTr("Ohm")],
-                ["\u03C9", qsTr("omega")],
-                ["\u00D7", qsTr("Multiplication")],
-                ["\u00F7", qsTr("Division")],
-                ["\u25FB", qsTr("Square")],
-                ["\u0394", qsTr("Delta")],
-                ["\\SA^B;", qsTr("Stacked text"), "\\\\SA^B;"]
+                    [0x00B0, qsTr("Degrees")],
+                    [0x00B1, qsTr("Plus/Minus")],
+                    [0x00F8, qsTr("Diameter")],
+                    [0x00F7, qsTr("Division")],
+                    [0x2248, qsTr("Almost Equal")],
+                    [0x2220, qsTr("Angle")],
+                    [0x2312, qsTr("Arc")],
+                    [0x2243, qsTr("Asymptotically Equal")],
+                    //[0xE100, qsTr("Boundary Line")],
+                    [0x2104, qsTr("Center Line")],
+                    [0x0394, qsTr("Delta")],
+                    [0x0278, qsTr("Electrical Phase")],
+                    //[0xE101, qsTr("Flow Line")],
+                    [0x2261, qsTr("Identity")],
+                    //[0xE200, qsTr("Initial Length")],
+                    //[0xE102, qsTr("Monument Line")],
+                    [0x00D7, qsTr("Multiplication")],
+                    [0x2260, qsTr("Not Equal")],
+                    [0x2126, qsTr("Ohm")],
+                    [0x03A9, qsTr("Omega")],
+                    [0x03C9, qsTr("omega")],
+                    [0x03A6, qsTr("Phi")],
+                    [0x0278, qsTr("phi")],
+                    [0x03C6, qsTr("Alt phi")],
+                    [0x03C0, qsTr("Pi")],
+                    [0x214A, qsTr("Property Line")],
+                    [0x2082, qsTr("Subscript 2")],
+                    [0x221A, qsTr("Square Root")],
+                    [0x25FB, qsTr("Square")],
+                    [0x00B2, qsTr("Squared")],
+                    [0x00B3, qsTr("Cubed")],
+                    ["\\SA^B;", qsTr("Stacked text"), "\\\\SA^B;"]
         ];
 
         if (dimension===true) {
@@ -1173,14 +1187,28 @@ WidgetFactory.initLineEdit = function(lineEdit, dimension) {
                 continue;
             }
 
-            var symbolSelf = symbol[0];
+            var symbolCode = symbol[0];
+            var symbolCodeStr;
+            var symbolSelf;
+            if (isNumber(symbolCode)) {
+                symbolCodeStr = symbolCode.toString(16);
+                while (symbolCodeStr.length<4) {
+                    symbolCodeStr = "0" + symbolCodeStr;
+                }
+                symbolCodeStr = ", \\U+" + symbolCodeStr;
+                symbolSelf = String.fromCharCode(symbolCode);
+            }
+            else {
+                symbolCodeStr = "";
+                symbolSelf = symbolCode;
+            }
             var symbolText = symbol[1];
             var insertion = symbol[0];
             if (symbol.length===3) {
                 insertion = symbol[2];
             }
 
-            var action = subMenu.addAction(symbolSelf + "\t(" + symbolText + ")");
+            var action = subMenu.addAction(symbolSelf + "\t(" + symbolText + symbolCodeStr + ")");
             eval("action.triggered.connect(function() { lineEdit.setFocus(Qt.OtherFocusReason); lineEdit.insert(\"" + insertion + "\"); });");
         }
 
